Program FComEdit;
{Frunze Commander's Text Editor}

{$X+,S-}

Uses
  CRT, DOS, Objects, Drivers, Memory, Views, Menus, Dialogs,
  StdDlg, MsgBox, App, Editors, Calc, Calendar;

Const
  HeapSize     = 2048;
  cmUserScreen = 1000;
  cmShowClip   =  100;
  cmCalculator =  102;
  cmCalendar   =  103;
  cmAbout      =  104;

Var
  X, Y  : Byte;
  Screen: Pointer;

type
  TVideoObj = object (TObject)
    Function VideoMemory: LongInt;
    Procedure SaveScreen;
    Procedure RestoreScreen;
  end;

  PTVClock = ^TTVClock;
  TTVClock = object (TView)
    Hour, Min, Sec : Word;
    Constructor Init (Var Bounds : TRect);
    Procedure Draw; Virtual;
    Procedure Update; Virtual;
  end;

  PEditorApp = ^TEditorApp;
  TEditorApp = Object (TApplication)
    V : TVideoObj;
    Clock : PTVClock;
    Constructor Init;
    Procedure HandleEvent (Var Event: TEvent); Virtual;
    Procedure InitMenuBar; Virtual;
    Procedure InitStatusLine; Virtual;
    Procedure UserScreen;
    Procedure Idle; virtual;
    Destructor Done; Virtual;
  End;

Var
  ClipWindow: PEditWindow;
  F         : Boolean;

Function TVideoObj.VideoMemory: LongInt;
Begin
  VideoMemory := SegB800;
  If Mem[Seg0040:$0049] = 7 then VideoMemory := SegB000
End;

Procedure TVideoObj.SaveScreen;
Begin
  GetMem (Screen, 8000);
  Move (Mem[VideoMemory:0], Screen^, 8000)
End;

Procedure TVideoObj.RestoreScreen;
Begin
  If Screen = nil then Exit;
  Move (Screen^,Mem[VideoMemory: 0], 8000)
End;

Constructor TTVClock.Init;
Begin
  Inherited Init (Bounds);
  Update
End;

Procedure TTVClock.Draw;
Var
  B : TDrawBuffer;
  C : Word;
  A, S : String;
Begin
  Str (Hour:2, A);
  S := A+':';
  Str (Min, A);
  If Length (A) = 1 then A := '0'+A;
  S := S+A+':';
  Str (Sec, A);
  If Length (A) = 1 then A := '0'+A;
  S := S+A;
  C := GetColor (1);
  MoveChar (B, ' ', C, Size.X);
  MoveStr (B, S, C);
  WriteLine (0, 0, Size.X, 1, B)
End;

Procedure TTVClock.Update;
Var
  H,M,S,T: Word;
Begin
  GetTime (H,M,S,T);
  If (Hour <> H) or (Min <> M) or (Sec <> S) then
    Begin
      Hour := H;
      Min := M;
      Sec := S;
      DrawView
    End
End;

Function OpenEditor (FileName: FNameStr; Visible: Boolean): PEditWindow;
Var
  P: PWindow;
  R: TRect;
Begin
  DeskTop^.GetExtent (R);
  P := New (PEditWindow, Init (R, FileName, wnNoNumber));
  If not Visible then P^.Hide;
  OpenEditor := PEditWindow (Application^.InsertWindow (P))
End;

Constructor TEditorApp.Init;
Var
  R : TRect;
  Reg : Registers;
Begin
  MaxHeapSize := HeapSize;
  Screen := nil;
  V.SaveScreen;
  With Reg do
    Begin
      AH := 3;
      BH := 0;
      Intr ($10, Reg);
      X := DL + 1;
      Y := DH + 1
    End;
  Inherited Init;
  GetExtent (R);
  Dec (R.B.X);
  R.A.X := R.B.X - 8;
  R.B.Y := R.A.Y + 1;
  Clock := New (PTVClock, Init (R));
  Insert (Clock);

  EditorDialog := StdEditorDialog;
    ClipWindow := OpenEditor ('', False);
  If ClipWindow <> nil then
    Begin
      ClipBoard := ClipWindow^.Editor;
      ClipBoard^.CanUndo := False;
    End;
  If F then
    Begin
      OpenEditor (ParamStr(1), True);
      F := False
    End
  Else
    OpenEditor ('', True)
End;

Procedure TEditorApp.Idle;
Begin
  If Clock <> nil then Clock^.Update;
  Inherited Idle
End;

Procedure TEditorApp.HandleEvent (Var Event: TEvent);

  Procedure FileOpen;
  Var
    FileName: FNameStr;
  Begin
    FileName := '*.*';
    If ExecuteDialog (New (PFileDialog, Init ('*.*', 'Open file',
      '~N~ame', fdOpenButton, 100)), @FileName) <> cmCancel then
      OpenEditor (FileName, True)
  End;

  Procedure FileNew;
  Begin
    OpenEditor ('', True)
  End;

  Procedure ChangeDir;
  Begin
    ExecuteDialog (New (PChDirDialog, Init (cdNormal, 0)), nil)
  End;

  Procedure ShowClip;
  Begin
    ClipWindow^.Select;
    ClipWindow^.Show
  End;

  Procedure Calculator;
  Begin
    InsertWindow (New(PCalculator, Init));
  End;

  Procedure Calendar;
  Begin
    InsertWindow (New(PCalendarWindow, Init));
  End;

  Procedure About;
  Begin
    MessageBox (^C'FCom Edit - Text Editor'^M^C'by Alexei A. Frounze'^M^C'(c) 2000',
                nil, mfInformation + mfOKButton);
  End;

Begin
  Inherited HandleEvent (Event);
  Case Event.What of
    evCommand:
      Case Event.Command of
        cmOpen      : FileOpen;
        cmNew       : FileNew;
        cmChangeDir : ChangeDir;
        cmShowClip  : ShowClip;
        cmUserScreen: UserScreen;
        cmCalculator: Calculator;
        cmCalendar  : Calendar;
        cmAbout     : About;
        Else Exit;
      End;
    Else Exit;
  End;
  ClearEvent (Event)
End;

Procedure TEditorApp.UserScreen;
Var C : Char;
Begin
  HideMouse;
  V.RestoreScreen;
  GotoXY (X, Y);
  C := ReadKey;
  If C = #0 then ReadKey;
  DoneMemory;
  Redraw;
  ShowMouse
End;

Procedure TEditorApp.InitMenuBar;
Var
  R: TRect;
Begin
  GetExtent (R);
  R.B.Y := R.A.Y + 1;
  MenuBar := New (PMenuBar, Init (R, NewMenu (
    NewSubMenu ('~F~ile', hcNoContext, NewMenu (
      NewItem ('~N~ew', '', kbNoKey, cmNew, hcNoContext,
      NewItem ('~O~pen...', 'F3', kbF3, cmOpen, hcNoContext,
      NewItem ('~S~ave', 'F2', kbF2, cmSave, hcNoContext,
      NewItem ('Save ~a~s', 'Shift+F2', kbShiftF2, cmSaveAs, hcNoContext,
      NewItem ('~C~hange dir...', '', kbNoKey, cmChangeDir, hcNoContext,
      NewLine (
      NewItem ('Calculator', 'Ctrl+F6', kbCtrlF6, cmCalculator, hcNoContext,
      NewItem ('Calendar', '', kbNoKey, cmCalendar, hcNoContext,
      NewLine (
      NewItem ('E~x~it', '', kbEsc, cmQuit, hcNoContext, nil))))))))))),
    NewSubMenu ('~E~dit', hcNoContext, NewMenu (
      StdEditMenuItems (
      NewLine (
      NewItem ('Show ~c~lipboard', 'Alt-C', kbAltC, cmShowClip, hcNoContext,
    nil)))),
    NewSubMenu('~S~earch', hcNoContext, NewMenu (
      NewItem ('~F~ind...', 'F7', kbF7, cmFind, hcNoContext,
      NewItem ('~S~earch again', 'Shift+F7', kbShiftF7, cmSearchAgain,
                                                                hcNoContext,
      NewItem ('~R~eplace...', '', kbNoKey, cmReplace, hcNoContext,
    nil)))),
    NewSubMenu ('~W~indows', hcNoContext, NewMenu (
      NewItem('~T~ile', '', kbNoKey, cmTile, hcTile,
      NewItem('C~a~scade', '', kbNoKey, cmCascade, hcCascade,
      NewLine(
      NewItem('~S~ize/Move','Ctrl+F5', kbCtrlF5, cmResize, hcResize,
      NewItem('~Z~oom', 'F5', kbF5, cmZoom, hcZoom,
      NewItem('~N~ext', 'F6', kbF6, cmNext, hcNext,
      NewItem('~P~revious', 'Shift+F6', kbShiftF6, cmPrev, hcPrev,
      NewItem('~C~lose', 'Alt+F3', kbAltF3, cmClose, hcClose,
    nil))))))))),
    NewSubMenu ('~H~elp', hcNoContext, NewMenu (
      NewItem('~A~bout...', 'F1', kbF1, cmAbout, hcNoContext,
    nil)),
  Nil))))))))
End;

Procedure TEditorApp.InitStatusLine;
Var
  R: TRect;
Begin
  GetExtent (R);
  R.A.Y := R.B.Y - 1;
  New (StatusLine, Init (R,
    NewStatusDef (0, $FFFF,
      NewStatusKey ('', kbF10, cmMenu,
      NewStatusKey ('~F2~ Save', kbF2, cmSave,
      NewStatusKey ('', kbAltX, cmQuit,
      NewStatusKey ('~F3~ Open', kbF3, cmOpen,
      NewStatusKey ('~Alt+F3~ Close', kbAltF3, cmClose,
      NewStatusKey ('~F7~ Find', kbF7, cmFind,
      NewStatusKey ('~Shift+F7~ Search again', kbShiftF7, cmSearchAgain,
      NewStatusKey ('~Alt+F5~ Screen', kbAltF5, cmUserScreen,
      NewStatusKey ('', kbAltC, cmShowClip,
    nil))))))))),
  nil)))
End;

Destructor TEditorApp.Done;
Begin
  Inherited Done;
  V.RestoreScreen;
  GotoXY (X, Y);
End;

Var
  EditorApp: TEditorApp;

Begin
  F := ParamCount = 1;
  EditorFlags := EditorFlags and ($FFFF-efBackupFiles);
  EditorApp.Init;
  EditorApp.Run;
  EditorApp.Done
End.
